######################################################################################################################
#                                                                                                                    #
#                                             Debug and logging utilities                                            #
#                                                                                                                    #
######################################################################################################################
active_debug_traces=''
print_traces() {
	local trace
	for trace; do
		if [[ " $active_debug_traces " =~ " $trace " ]]; then continue; fi
		active_debug_traces="$active_debug_traces $trace"
	done
	active_debug_traces="${active_debug_traces# }"
}

noprint_traces() {
	local trace
	active_debug_traces=" ${active_debug_traces# } "
	for trace; do
		active_debug_traces="${active_debug_traces/ $trace / }"
	done
	active_debug_traces="${active_debug_traces% }"
	active_debug_traces="${active_debug_traces# }"
}

dbg() {
	local trace_channel="$1"
	shift
	if [ -z $active_debug_traces ]; then return 0; fi
	if [[ " $active_debug_traces " =~ " $trace_channel " ]]; then
		echo "${trace_channel^^}:" "$@" >&2
	fi
}

dbg_cat() {
	local trace_channel="$1"
	local message="$2"
	local file="$3"
	local fsize
	if [ -z "$file" -a -r "$message" ]; then
		file="$message"
		message="Displaying"
	fi
	
	if [ -z $active_debug_traces ]; then return 0; fi
	if [[ " $active_debug_traces " =~ " $trace_channel " ]]; then
		echo "${trace_channel^^}: $message '$file':"
		if [ -s "$file" ]; then
			fsize=$(wc -l "$file")
			if [ ${fsize%% *} -gt 25 ]; then
				head "$file"
				echo "        ........"
				tail "$file"
			else
				cat "$file"
			fi
		elif [ -r "$file" ]; then
			echo "	<empty>"
		else
			echo "	<FILE NOT FOUND>"
		fi
		echo ""
	fi >&2
}

: ${quiet:=false}
log() {
	$quiet || echo "$@" >&2
}

log_vars() {
	local var
	for var; do
		echo "$var='${!var}'"
	done
}

internal_error() {
	local status="$1"
	shift
	local message="$*"
	if [ -z "$message" ]; then
		message="$status"
		status=1
	fi
	echo -n "INTERNAL ERROR: $message at " >&2
	caller 1 >&2
	local i=2
	while caller $((i++)); do true; done | sed -e 's/^/	Called from /' >&2
	exit $status
}

fatal() {
	local status="$1"
	shift
	local message="$*"
	if [ -z "$message" ]; then
		message="$status"
		status=1
	fi
	echo -n "ERROR: $message" >&2
	exit $status
}

error() {
	echo -n "ERROR: $*" >&2
}

warning() {
	echo "WARNING: $*" >&2
}

#
# Execute a command, logging it if needed, and exit in case of error
#
log_exec() {
	$quiet || echo "$@" >&2
	eval "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		error $status "Command '$@' failed, status=$status..." >&2
	fi
}

silent_exec() {
	eval "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		error $status "Command '$@' failed, status=$status..." >&2
	fi
}
