#
# Utilities for building and installing native packages in the native sysroot
#

# Script to adapt native scripts to work from the sysroot
: ${ignored_paths:="/usr/include /lib /usr/lib /usr/local/lib /bin /usr/bin /sbin /usr/sbin /usr/local/bin /usr/local/sbin"}

adapt_native_scripts() {
	log "Adapting native scripts to work in sysroot '$SYSROOT_LOC'"
	
	local native_referenced_paths=
	local files_to_adapt=
	local referenced_paths file path from what
	
	referenced_paths=$(grep -r /usr/ _install \
					 | grep -v '^Binary file' \
					 | sed -e 's|^_install/\([^:]*\):.*\(/usr/[-_.a-zA-Z0-9/]*\).*$|/\1:\2|' \
					 | sort | uniq)
	for path in $referenced_paths; do
		from=${path%:*}
		what=${path#*:}
		if [ -f "${NATIVE_SYSROOT_LOC%/}/${from#/}" ]; then
			if [ -r "${NATIVE_SYSROOT_LOC%/}/${what#/}" ]; then
				if [[ " $ignored_paths " =~ " ${what%/} " ]]; then
					continue
				fi
				if [[ " $native_referenced_paths " =~ " $what " ]]; then
					true # already referenced
				else
					native_referenced_paths="$native_referenced_paths $what"
				fi
			else
				continue
			fi
			if [[ " $files_to_adapt " =~ " $from " ]]; then
				true # already referenced
			else
				files_to_adapt="$files_to_adapt $from"
			fi
		fi
	done
	
	local sed_file=/tmp/adapt.$$.sed
	touch $sed_file
	for what in $native_referenced_paths; do
		echo 's|\([^a-zA-Z0-9]\)'"$what"'|\1'"${NATIVE_SYSROOT_LOC%/}"'/'"${what#/}"'|g' >> $sed_file
		echo 's|^'"$what"'|'"${NATIVE_SYSROOT_LOC%/}"'/'"${what#/}"'|' >> $sed_file
	done
	
	for file in $files_to_adapt; do
		path="$NATIVE_SYSROOT_LOC/${file#/}"
		if [ -r "$path" ]; then
			log "	Adapting '$file'"
			rm -f $path.bak
			cp $path $path.bak
			sed -f $sed_file $path.bak > $path
			rm -f $path.bak
		else
			log "Can't find '$file' in '$NATIVE_SYSROOT_LOC'; Abort"
			exit 1
		fi
	done
	
	rm -f $sed_file
}

# Generate a native .pc file, as for a cross-compiled one :-)
require_once package_utils.sh
generate_native_pc_file() {
	generate_pc_file "$@"
}