######################################################################################################################
#                                                                                                                    #
#                                                  PACKAGE configuration                                             #
#                                                                                                                    #
######################################################################################################################

# Package-specific variables
package_id=$recipe_id
package_name=$recipe_name
package_version=$recipe_version
package_revision=$recipe_revision
package_arch=${PACKAGE_ARCH//_/-/}

# Default values
: ${package_arch:=all}

# Package-specific derived variables
package_suffix=${recipe_version}-${recipe_revision}_${package_arch}

######################################################################################################################
#                                                                                                                    #
#                                              Layout of the feeds directory                                         #
#                                                                                                                    #
######################################################################################################################

#
#	feeds
#		contents
#			$arch_dir_name
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#			native
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#			all
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#		metadata
#			$arch_dir_name
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					$bin_package.rdepends
#					$bin_package.rrecommends
#					$bin_package.rsuggests
#					$bin_package.rconflicts
#					$bin_package.rreplaces
#					$bin_package.summary
#					$bin_package.description
#					$bin_package.section
#					$bin_package.homepage
#					$bin_package.license
#					$bin_package.preinst
#					$bin_package.postinst
#					$bin_package.prerm
#					$bin_package.postrm
#			native
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					...
#			all
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					...
#		packages
#			$arch_dir_name
#				$bin_package					// ID of the generating recipe
#			native
#				$bin_package					// ID of the generating recipe
#			all
#				$bin_package					// ID of the generating recipe
#		tarballs
#			$arch_dir_name
#				$recipe_id
#					$bin_package.tar.bz2		// Installable binary package tarball (without metadata)
#			native
#				$recipe_id
#					$bin_package				// Installable binary package tarball (without metadata)
#			all
#				$recipe_id
#					$bin_package				// Installable binary package tarball (without metadata)
#		ipk
#			$arch_dir_name
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#			native
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#			all
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#

feedsDir=$PLATFORM_LOC/feeds

# Sub-directories where to place generated files
metadataSubDir=metadata/$package_arch/$package_id
contentsSubDir=contents/$package_arch/$package_id
packagesSubDir=packages/$package_arch
tarballsSubDir=tarballs/$package_arch/$package_id
ipkSubDir=${feedsDir}/ipk/$package_arch

# Package-dependent feed directories
metadataDir=${feedsDir}/$metadataSubDir
contentsDir=${feedsDir}/$contentsSubDir
packagesDir=${feedsDir}/$packagesSubDir
tarballsDir=${feedsDir}/$tarballsSubDir
ipkDir=${feedsDir}/${ipkSubDir}

# Package-independent feed sub-directories
allMetadataSubDir=metadata/$package_arch
allContentsSubDir=contents/$package_arch
allPackagesSubDir=packages/$package_arch
allTarballsSubDir=tarballs/$package_arch
allIpkSubDir=${ipkSubDir}

# Package-independent feed directories
allMetadataDir=${feedsDir}/$allMetadataSubDir
allContentsDir=${feedsDir}/$allContentsSubDir
allPackagesDir=${feedsDir}/$allPackagesSubDir
allTarballsDir=${feedsDir}/$allTarballsSubDir
allIpkDir=${feedsDir}/$allIpkSubDir

# Architecture-independent feed directories
baseMetadataDir=${feedsDir}/metadata
baseContentsDir=${feedsDir}/contents
basePackagesDir=${feedsDir}/packages
baseTarballsDir=${feedsDir}/tarballs
baseIpkDir=${feedsDir}/ipk

######################################################################################################################
#                                                                                                                    #
#                                                 Package-name utilities                                             #
#                                                                                                                    #
######################################################################################################################

# Splits a full package name in 
#	package-name
#	version
#	revision
#	architecture
# separated by the provided character (colon by default)
split_package_name() {
	local separator="${2:-:}"
	full_package_name "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)_\(.*\)$/\1'$sep'\2'$sep'\3'$sep'\4/'
}

# Splits a package id in 
#	package-name
#	version
#	revision
# separated by the provided character (colon by default)
split_package_id() {
        local separator="${2:-:}"
        echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)$/\1'$sep'\2'$sep'\3/'
}

# Returns a package ID from a package name
package_id() {
	echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\).*$/\1_\2-\3/'
}

# Returns a package architecture from a package name
package_arch() {
	echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)\_\(.*\)$/\4/'
}

# Compute the name of a package-specific variable
package_var_name() {
	local package=${1%%_*}
	package=${package//-/_}
	package=${package//+/P}
	local variable="$2"
	echo ${variable}_${package}
}

# Get the value of a package-specific variable
# Here I may add support for overrides, if it becomes needed...
package_var() {
	local name=$(package_var_name "$@")
	local prepend="${name}_prepend"
	local append="${name}_append"
	echo ${!prepend}${!name}${!append}
}

package_contents() {
	local pkg="$1"
	local contents="$(echo ${contentsDir}/${pkg}*.list)"
	if [ -r "$contents" ]; then 
		echo "$contents"
	fi
}

full_package_name() {
	local pkg="$1"
	local needed="$2"
	local contents="$(package_contents $pkg)"
	if [ -n "$contents" ]; then
		basename "$contents" .list
	elif [ -n "$needed" ]; then
		echo "Package '$pkg' was not built" >&2
		return 1
	fi
}

package_depends() {
	local pkg="$1"
	local how="${2-RDEPENDS}"
	(
		local dep auto_dep=${how,,}
		local depends=$(echo ${metadataDir}/$pkg*/$pkg*.auto-${auto_dep#r})
		if [ -s "$depends" ]; then
			for dep in $(cat "$depends"); do
				echo $dep
			done
		fi
		for dep in $(package_var $pkg ${how^^}); do
			echo $dep
		done
	) | sort | uniq
}

######################################################################################################################
#                                                                                                                    #
#                                                 Pkg-config generation                                              #
#                                                                                                                    #
######################################################################################################################

pc_tag_vars="
	Name:recipe_name
	Description:SUMMARY
	Version:recipe_version_revision
	Requires:recipe_buildtime_dependencies
	Conflicts:RCONFLICTS
"

# If there are specific libraries to include, add their names to pc_additional_libs
# If specific C flags must be provided, specify them in pc_additional_cflags
# You don't need to add -L or -I for your package, they are automatically provided
pc_additional_libs=
if [[ " $recipe_name " =~ " lib*" ]]; then
	pc_additional_libs=$recipe_name
fi
pc_additional_cflags=

#
# Generate a .pc file for the current recipe
#
# If the recipe has defined non-standard installation directories that may be used
# in Cflags or Libs, give their names in $non_standard_install_dir_vars
#
# By default, if a library was created having the name of the recipe, a -l link option is generated
# If additional libraries must also be linked in depending packages, define them in $pc_additional_libs
#
# If additional variables must be included in the .pc file
# their NAMES must be provided as parameters
#

generate_pc_file_vars() {
	local vars_file="$1"
	shift
	local all_vars="$*"
	local var value exec_values data_values
	
	for var in $all_vars; do
			value="${!var}"
			if [ -z "$value" ]; then continue; fi
			case $var in
			*bindir | *libdir | *libexecdir)
#				if [ ! -d "_install/$value" ]; then continue; fi 
				tac $vars_file.exec.sed > $vars_file.exec.rev.sed
				echo "$var=$value" | sed -f $vars_file.exec.rev.sed
				if [[ " $exec_values " =~ " $value " ]]; then continue; fi
				exec_values="$exec_values $value"
				echo '\|^'"$var"'=|!s|'"$value/"'|${'"$var"'}/|' >> $vars_file.exec.sed.rev
				echo '\|^'"$var"'=|!s|'"$value"'$|${'"$var"'}|' >> $vars_file.exec.sed.rev
				;;
			*dir)
#				if [ ! -d "_install/$value" ]; then continue; fi 
				tac $vars_file.data.sed > $vars_file.data.rev.sed
				echo "$var=$value" | sed -f $vars_file.data.rev.sed
				if [[ " $data_values " =~ " $value " ]]; then continue; fi
				data_values="$data_values $value"
				echo '\|^'"$var"'=|!s|'"$value/"'|${'"$var"'}/|' >> $vars_file.data.sed
				echo '\|^'"$var"'=|!s|'"$value"'$|${'"$var"'}|' >> $vars_file.data.sed
				;;
			*)
				echo "$var=$value"
				;;
		esac
	done
}

generate_pc_file() {
	local pc_file="${1:-$recipe_name}.pc"
	local pc_dir="_install/usr/lib/pkgconfig"
	local pkg_var=$(shell_variable_name $recipe_name)
	local vars_file=$tmpdir/all_pc_vars.$$
	local additional_vars="$*"
	local var var_name spec tag spec_var value val normalized_value lib
	
	# First take into account the case where the pc_file passed is absolute
	case $pc_dir in
		/*)	pc_dir=_install${pc_file%/*}
			pc_file=${pc_file##*/}
			;;
	esac
	mkdir -p "$pc_dir"
	pc_file="$pc_dir/$pc_file"
	rm -f $vars_file.*

	for var in prefix exec_prefix; do
		echo '\|^'"$var"'=|!s|'"${!var}/"'|${'"$var"'}/|'
		echo '\|^'"$var"'=|!s|'"${!var}"'$|${'"$var"'}|'
	done >> $vars_file.exec.sed
	for var in exec_prefix prefix; do
		echo '\|^'"$var"'=|!s|'"${!var}/"'|${'"$var"'}/|'
		echo '\|^'"$var"'=|!s|'"${!var}"'$|${'"$var"'}|'
	done >> $vars_file.data.sed
	
	(
		# Now generate the file
		echo "# Standard directories" 
		echo "prefix=${prefix}" 
		if [ "$prefix" = "$exec_prefix" ]; then
			echo "exec_prefix=\${prefix}" 
		else
			value=" $exec_prefix"
			value=${value/ $prefix\// \$prefix/}
			value=${value# }
			echo "exec_prefix=${value}" 
		fi
		generate_pc_file_vars $vars_file $install_dir_vars
		
		if [ -n "$non_standard_install_dir_vars" ]; then
			echo # Separate standard and non-standard variables 
			echo "# Nonstandard directories"
			generate_pc_file_vars $vars_file $non_standard_install_dir_vars
		fi
		
		if [ -n "$additional_vars" ]; then
			echo # Separate standard and non-standard variables 
			echo "# Additional variables"
			generate_pc_file_vars $vars_file $additional_vars
		fi
	
		echo # Also separate description from variables
		echo "# Package description"
		for spec in $pc_tag_vars; do
			var=$(shell_variable_name ${spec#*:})
			tag=${spec%%:*}
			value=
			var_name=$var
			if [ "${!var_name+set}" ]; then
				value="${!var_name}"
			else
				var_name=$(shell_variable_name ${var}_${recipe_name})
				if [ "${!var_name+set}" ]; then
					value="${!var_name}"
				fi
			fi
			if [ -n "${value}" ]; then
				if [[ " $var " =~ "_dependencies " || " $var " =~ " R" ]]; then
					normalized_value=
					for val in $value; do
						case $val in
							\(*)	
								normalized_value="$normalized_value ${val#(}"
								;;
							*\))	
								normalized_value="$normalized_value ${val%)}"
								;;
							*_*)	
								normalized_value="$normalized_value ${val%_*} >= ${val#*_}"
								;;
							*)
								normalized_value="$normalized_value ${val}"
								;;
						esac
					done
					echo "${tag}:${normalized_value}"
				else
					echo "${tag}: ${value}"
				fi
			fi
		done
		
		# Adds the includedir to the Cflags
		if [ -n "$includedir" -a -d "_install/$includedir" ]; then
			normalized_value="-I\${includedir}"
			if [ -n "pc_additional_cflags" ]; then
				normalized_value="$normalized_value $pc_additional_cflags"
			fi
		else
			normalized_value="$pc_additional_cflags"
		fi
		if [ -n "$normalized_value" ]; then
			echo "Cflags: ${normalized_value# }"
		fi
	
		# Adds the libdir to the Libs and normalize library references
		if [ -n "$libdir" -a -d "_install/$libdir" ]; then
			normalized_value="-L\${libdir}"
		fi
		for lib in $recipe_name $pc_additional_libs; do
			lib=${lib#lib}
			lib=${lib%.a}
			lib=${lib%.so}
			value="$(find _install -name lib${lib}.so)$(find _install -name lib${lib}.a)"
			if [ -n "$value" ]; then
				normalized_value="$normalized_value -l${lib}"
			fi
		done
		if [ -n "$normalized_value" ]; then
			echo "Libs: ${normalized_value# }"
		fi
	) >> $pc_file
	rm -f $vars_file.*
}