######################################################################################################################
#                                                                                                                    #
#                                Generic script routines usable in generation scripts                                #
#                                                                                                                    #
######################################################################################################################

here=$(pwd)
tmpdir=/tmp

#
# Before everything, switch to common "C" language locale to avoid localization uglinesses
# If at any moment we really need local language support, we could restore these
#
ORIGINAL_LOCALE_LANG="$LANG";           export ORIGINAL_LOCALE_LANG
ORIGINAL_LOCALE_LANGUAGE="$LANGUAGE";   export ORIGINAL_LOCALE_LANGUAGE
ORIGINAL_LOCALE_LC_ALL="$LC_ALL";       export ORIGINAL_LOCALE_LC_ALL
ORIGINAL_LOCALE_LC_CTYPE="$LC_CTYPE";   export ORIGINAL_LOCALE_LC_CTYPE
LANG=C;                                 export LANG
LANGUAGE=C;                             export LANGUAGE
LC_ALL=C;                               export LC_ALL
LC_CTYPE=C;                             export LC_CTYPE

#
# Ensure we get shared library from the native sysroot before the installed ones or sysroot-installed
# native tools may not work as expected...
#
for dir in /usr/lib /lib $(cat $NATIVE_SYSROOT_LOC/etc/ld.so.conf 2>/dev/null); do
	if [ -d "$NATIVE_SYSROOT_LOC$dir" ]; then
		LD_LIBRARY_PATH="$NATIVE_SYSROOT_LOC$dir:$LD_LIBRARY_PATH"
	fi
done
export LD_LIBRARY_PATH

#
# First determine the current buildstep and script name
#
current_buildstep=$CURRENT_BUILDSTEP
buildstep_script=$BUILDSTEP_SCRIPT
if [ -z "$buildstep_script" ]; then
	buildstep_script=$(basename "$0" .sh)
	buildstep_script=${buildstep_script#do_}
fi
if [ -z "$current_buildstep" ]; then
	case $buildstep_script in
		make_all)		current_buildstep=compile			;;
		make_install)	current_buildstep=install			;;
		*)				current_buildstep=$buildstep_script	;;
	esac
fi
current_task="$current_buildstep" # Task and Buildstep are synonyms

#
# Transform a entity name in a shell variable name
#
shell_variable_name() {
	local name="$1"
	name="${name//+/P}"
	name="${name//[^a-zA-Z0-9_]/_}"
	echo "$name"
}
	
#
# Package IDs are formed as follows:
#
#	name_vers-rev_arch
#
# where:
#	name	name of the package; it could contain '-' but no '.' or '_'
#	vers	version of the package; it could contain '.' but no '-' or '_'
#	rev	revision o fthe package build rules; it could contain '-' or '.' but no '_'
#
# Built binary package names follow the pattern
#
#	name-sub_vers-rev_arch
#
# where:
#	sub	sub-package; it could contain '.' but no '-' or '_'
#	arch	architecture for which the package was built; it could contain '-' but no '.' or '_'
# (others are as above)
#

######################################################################################################################
#                                                                                                                    #
#                                                 PLATFORM configuration                                             #
#                                                                                                                    #
######################################################################################################################

# Platform-specific variables
platform_loc=$PLATFORM_LOC
platform_id=$(basename $platform_loc)
platform_name=${platform_id%%_*}
platform_version=${platform_id#*_}

# These variables should be provided by the platform...
: ${arch_compiler_prefix:=${TARGET_SYSROOT_LOC##*/}}
: ${generic_subpackages:=""}
: ${global_additional_subpackages:=""}

ac6_platform_manifest=$platform_loc/ac6_platform_manifest.xml
ac6_platform_env=$platform_loc/sysroots/$arch_compiler_prefix/etc/ac6_platform.env

if [ $ac6_platform_manifest -nt $ac6_platform_env ]; then
	mkdir -p $(dirname $ac6_platform_env)
	rm -f $ac6_platform_env
	tmpbase=$tmpdir/ac6_platform_env.$$
	cat << EOF > $tmpbase.sed
s/^.*<target_triplet>\(.*\)<\/target_triplet>.*$/compiler_prefix="\1"/p
/<property name="arch_compiler_prefix".*>$arch_compiler_prefix<\/property>/,/<\/properties>/{
s/^.*<property name="\([a-zA-Z0-9_]\+\)"[^>]*>\(.*\)<\/property>.*$/\1="\2"/p
}
EOF
	sed -n -f $tmpbase.sed $ac6_platform_manifest > $ac6_platform_env
fi
if [ ! -r $ac6_platform_env ]; then
	echo "No platform found... Bail out!" >&2
	exit 1
fi
. $ac6_platform_env


######################################################################################################################
#                                                                                                                    #
#                                                  RECIPE configuration                                              #
#                                                                                                                    #
######################################################################################################################

# Recipe specific variables
recipe_id=$RECIPE_ID # Can't use directly as version/revision not at the proper place for sub-packages
recipe_name=$RECIPE_NAME
recipe_version=$RECIPE_VERSION
recipe_revision=$RECIPE_REV

# Backward compatibility settings
: ${recipe_id:=$PACKAGE_ID} # Can't use directly as version/revision not at the proper place for sub-packages
: ${recipe_name:=$PACKAGE_NAME}
: ${recipe_version:=$PACKAGE_VERSION}
: ${recipe_revision:=$PACKAGE_REV}

# Default values
: ${recipe_revision:=r0}
: ${recipe_statedir:=.ac6_build}

# Recipe-specific derived variables
recipe_version_revision=${recipe_version}-${recipe_revision}
recipe_workdir="$taskBuildStatusDir"
#recipe_workdir="$WORKSPACE_LOC/$recipe_id" # Only valid after the Import buildstep // TODO: export from the IDE

# Create and source the bash-compatible recipe manifest
#
# This defines various variables that should be exported by the IDE:
#	recipe_kind
#	recipe_classes
#	recipe_metadatadir
#	recipe_runtime_dependencies
#	
# FIXME: export these from the IDE
#
recipe_manifest=$RECIPE_MANIFEST
if [ -z "$recipe_manifest" ]; then
	for recipe_kind in packages kernels rootfs; do
		recipe_metadatadir="$platform_loc/metadata/$recipe_kind/$recipe_id"
		recipe_manifest="$recipe_metadatadir/ac6_recipe_manifest.xml"
		if [ -r "$recipe_manifest" ]; then
			break;
		fi
		recipe_manifest="$recipe_metadatadir/ac6_package_manifest.xml"
		if [ -r "$recipe_manifest" ]; then
			break;
		fi
	done
fi
if [ -n "$recipe_manifest" ]; then
	recipe_metadatadir=$(dirname "$recipe_manifest")
fi
if [ -d "$recipe_workdir" ]; then
	mkdir -p "$recipe_workdir/.ac6_build"
	recipe_env="$recipe_workdir/.ac6_build/recipe_manifest.env"
else
	# We are in Fetch, before workdir creation, and we should not create it, as Import may choke at it...
	recipe_env="$tmpdir/recipe_manifest.env.$$"
fi
if [ -n "$recipe_manifest" ]; then
	if [ "$recipe_manifest" -nt "$recipe_env" -o "$recipe_buildsteps" -nt "$recipe_env" ]; then
		rm -f $recipe_env
		tmpbase=$tmpdir/utils.$$
		cat << EOF > $tmpbase.sed
/^.*<typeId>fr\.ac6\.platform\..*<\/typeId>.*$/{
s/^.*<typeId>fr\.ac6\.platform\.\(.*\)<\/typeId>.*$/\1/
s/\./ /g
p
}
EOF
		recipe_classes=
		for i in $(sed -n -f $tmpbase.sed $recipe_manifest); do
			recipe_classes="$i $recipe_classes"
		done
		cat << EOF > $tmpbase.sed
s/^.*<dependency.*type="runtime".*>\(.*\)<\/dependency>.*$/\1/p
EOF
			recipe_runtime_dependencies=
			for i in $(sed -n -f $tmpbase.sed $recipe_manifest); do
				recipe_runtime_dependencies="$recipe_runtime_dependencies $i"
			done
			cat << EOF > $tmpbase.sed
s/^.*<dependency.*type="buildtime".*>\(.*\)<\/dependency>.*$/\1/p
EOF
		recipe_buildtime_dependencies=
		for i in $(sed -n -f $tmpbase.sed $recipe_manifest); do
			recipe_buildtime_dependencies="$recipe_buildtime_dependencies $i"
		done
		echo "recipe_kind='${recipe_kind}'"                                     > $recipe_env
		echo "recipe_classes='${recipe_classes%% }'"                            >> $recipe_env
		echo "recipe_runtime_dependencies='${recipe_runtime_dependencies## }'"  >> $recipe_env
		echo "recipe_buildtime_dependencies='${recipe_buildtime_dependencies## }'"  >> $recipe_env
	fi
else
	echo "ERROR: Can't find recipe manifest for '$recipe_id' in platform '$(basename $platform_loc)'"
	exit 1
fi
if [ -r $recipe_env ]; then
	source $recipe_env
fi

# Package where package install step place files
: ${installDir:=$here/_install}

# Package installation directories in installDir
: ${prefix:="/usr"}
: ${exec_prefix:="${prefix}"}
: ${bindir:="${exec_prefix}/bin"}
: ${sbindir:="${exec_prefix}/sbin"}
: ${libdir:="${exec_prefix}/lib"}
: ${libexecdir:="${exec_prefix}/libexec"}
: ${datarootdir:="${prefix}/share"}
: ${datadir:="${datarootdir}"}
: ${sysconfdir:="${prefix}/etc"}
: ${pkgconfigdir:="${libdir}/pkgconfig"}
: ${localstatedir:="${prefix}/var"}
: ${docdir:="${datarootdir}/doc/${recipe_name}"}
: ${mandir:="${datarootdir}/man"}
: ${infodir:="${datarootdir}/info"}
: ${includedir:="${prefix}/include"}
: ${localedir:="${datarootdir}/locale"}

#
# The variables below are used for pkg-config PC file generation
#

# If a recipe adds new values there, it should NOT expand this list but add them
# to the non_standard_dir_vars; these must be listed in the order they are defined.
# Note that prefix is NOT in the list as it is usually the base for all others
install_dir_vars="bindir sbindir libdir libexecdir datarootdir datadir sysconfdir \
				  localstatedir docdir mandir infodir includedir localedir"
non_standard_install_dir_vars=

######################################################################################################################
#                                                                                                                    #
#                                                   Task configuration                                               #
#                                                                                                                    #
######################################################################################################################

: ${current_task_id=task.${current_task}.${recipe_id}}
case $current_task in
	populate-sysroot)
		current_task_statedir="${SYSROOT_LOC}/${recipe_statedir}"
		;;
	split | package)
		current_task_statedir="${PLATFORM_LOC}/feeds/${recipe_statedir}"
		;;
	fetch)
		current_task_statedir="${DOWNLOAD_LOC}/${recipe_statedir}"
		;;
	*)
		current_task_statedir="${recipe_workdir}/${recipe_statedir}"
		;;
esac
current_task_statefile="$current_task_statedir/${current_task_id}"

######################################################################################################################
#                                                                                                                    #
#                                                    Script utilities                                                #
#                                                                                                                    #
######################################################################################################################

#
# Search and source recipe-dependent scripts
#
# All these functions will search for a script in various directories:
#		The platform package (or kernel/platform/rootfs)-specific buildstep script directory
#		The platform generic script directory
#		The IDE default script library directory
# 	The script SCRIPT is searched (in order) as SCRIPT SCRIPT.sh do_SCRIPT do_SCRIPT.sh
#
# There is five functions in this group
#
#		search_script			search for the required script and outputs its filepath if found
#		source_script			search for the required script and sources it if found
#		require_script			search for the required script and sources it but exit in error if not found
#		do_buildstep_prepend	search and execute a buildstep prepend script, if there is any
#		do_buildstep_append		search and execute a buildstep append script, if there is any
#
# In all buildstep scripts, the prepend script will be sourced after the environment was set up (variables, 
# working directories, etc.) but before any real work was done.
#
# The append script will be sourced at the real end of the buildstep script, just before its result is
# installed in the global directory if it's not a recipe-local buildstep
#
# All scripts sourced during a task are automatically hashed and the hash saved 
# (sorted) in an includes file so that changing a file may invalidate the task
#
sw4l_install_base=${script_lib_dir%/plugins/fr.ac6.*}
if [ -d "${current_task_statedir}" ]; then
	loaded_scripts_state=${current_task_statedir}/${current_task_id}.includes
else # Project not yet created by Import, just place sate file in some temporary directory
	loaded_scripts_state=$tmpdir/${current_task_id}.$$.includes
fi
rm -f "$loaded_scripts_state"
touch "$loaded_scripts_state"

# Parameter is the level in call stack from where we want to get the file name
# Passing 0 (or nothing) will return the filename of the call, 1 the one of its
# caller (assuming the call is done in a function) a.s.o
current_script_file() {
	local depth=${1:-0}
    local sourced="${BASH_SOURCE[$depth + 1]}"
# Unnecessary, as BASH_SOURCE contains only absolute path names (even if
# accessed through the PATH)
#    case "$sourced" in
#    	/*) ;;
#    	*)	sourced="$(pwd)/${sourced#./}" ;;
#    esac
	echo "$sourced"
}

task_execute_date() {
	date -u +"%F %T %N"
}
task_start_date="$(task_execute_date)"

record_script_load() {
	local script_file="${1:-$(current_script_file 1)}"
# For now keep IDE-provided scripts in the checksums...
#	if [[ " $script_file " =~ " $sw4l_install_base/" ]]; then
#		return # Don't record scripts taken from the IDE itself, they don't change
#	fi

	# Only record a script once, even if loaded several times
	if [[ " $already_loaded_scripts " =~ " $found " ]]; then
		# Script was already loaded, so recorded, just return 1 so that caller knows that
		return 1
	fi
	already_loaded_scripts="$already_loaded_scripts $found"

	rm -f "$loaded_scripts_state.bak"
	mv "$loaded_scripts_state" "$loaded_scripts_state.bak"
	echo "$script_file" >> "$loaded_scripts_state.bak"
	sort "$loaded_scripts_state.bak" | uniq > "$loaded_scripts_state"
	rm -f "$loaded_scripts_state.bak"
	return 0
}
record_script_load $BASH_SOURCE

setup_scripts_dir() {
	local class dir
	dir="$recipe_metadatadir/buildsteps/scripts"
	if [ -d "$dir" ]; then
		echo "$dir"
	fi
	dir="$PLATFORM_LOC/scripts"
	if [ -d "$dir" ]; then
		for class in $recipe_classes; do
			if [ -d "$dir/$class" ]; then
				echo "$dir/$class"
			fi
		done
		echo "$dir"
	fi
	if [ -d "$script_lib_dir" ]; then
		echo "$script_lib_dir"
	fi
}
[ -n "$scripts_search_path" ] || scripts_search_path=$(setup_scripts_dir)

search_script() {
	local script scriptdir
	for script; do
		for scriptdir in $scripts_search_path; do
			for file in $script $script.sh do_$script do_$script.sh; do
				if [ -r "$scriptdir/$file" ]; then
					echo "$scriptdir/$file"
					return
				fi
			done
		done
	done
}

do_require_script() {
	local scripts="$*"
	local found=$(search_script "$@")
	if [ -z "$found" ]; then
		local message="Cannot find any of scripts"
		declare -a script
		read -a script <<< "$scripts"
		local sep i
		for ((i=0; i < ${#script[@]} - 1; i++)) do
			message="$message$sep '${script[i]}'"
			sep=","
		done
		if [ -n "$sep" ]; then sep=" or"; fi
		message="$message$sep '${script[i]}'"
		# Can't use error here: this function is needed to load log_utils.sh...
		echo "ERROR: $message" >&2
		caller 1
		caller 2
		caller 3
		exit 1
	fi
	echo "$found"
}

require_script_once() {
	local found=$(do_require_script "$@")
	if record_script_load "$found"; then
		source "$found"
	else
		true # Script was already loaded, skip it
	fi
}

require_script() {
	local found=$(do_require_script "$@")
	record_script_load "$found"
	source "$found"
}

source_script_once() {
	local found=$(search_script "$@")
	if [ -z "$found" ]; then
		return 0 # Not finding the script is NOT an error, it's expected
	fi
	if record_script_load "$found"; then
		source "$found"
	else
		true # Script was already loaded, skip it
	fi
}

source_script() {
	local found=$(search_script "$@")
	if [ -z "$found" ]; then
		return 0 # Not finding the script is NOT an error, it's expected
	fi
	record_script_load "$found"
	source "$found"
}

do_buildstep_prepend() {
	source_script $buildstep_script-prepend.sh $current_buildstep-prepend.sh
}

do_buildstep_append() {
	source_script $buildstep_script-append.sh $current_buildstep-append.sh
}

######################################################################################################################
#                                                                                                                    #
#                                          Source specific script routines                                           #
#                                                                                                                    #
######################################################################################################################

require_script log_utils.sh
require_script package_utils.sh
require_script path_utils.sh
require_script task_utils.sh
require_script make_utils.sh

# Also include buildstep-specific utilisties, if any
source_script ${current_buildstep}_utils.sh 
source_script ${buildstep_script}_utils.sh 

######################################################################################################################
#                                                                                                                    #
#                                           End of generic script routines                                           #
#                                                                                                                    #
######################################################################################################################
