#! /bin/bash
#
# Compute the combined script hash for a given task of a given recipe
#
# This script needs at least 3 parameters:
#	The platform				/home/.../minimal-platform_1.0.0
#	The arch_compiler_prefix	cortexa9hf-vfp-neon or native
#	The task ID(s)				task.populate-sysroot.qt_5.7.0
# For each provided task ID, a task_state.script-hash file is generated, 
# containing the hashes of all used files; this file may then be
# incorporated in the global task hash.
#
# The environment should also provide:
#	the SYSROOT_LOC
#

#################################################################
#                                                               #
# This was adapted from task-utils.sh -- keep them synchronized #
#                                                               #
#################################################################
: ${recipe_statedir:=.ac6_build}

platform_loc="$1"
arch_compiler_prefix="$2"
task_id="$3"

task_start_date="$(date -u +"%F %T %N")"
workspace=${platform_loc%/*}
sysroot_loc=$workspace/sysroots/$arch_compiler_prefix

compute_script_sum() {
	local script_file="$1"
	if [ -r "$script_file" ]; then
		#
		# Normalize scripts before computing checksum:
		#	Eliminate all comments
		#	Replace all space sequences by a single space
		#	Eliminate leading and trailing spaces
		#	Eliminate empty lines
		#
		echo -n "${script_file##*/}:"
		sed -e '/^#/d' \
		    -e 's/#.*$//' \
		    -e 's/[[:space:]]\+/ /' \
		    -e 's/^ //' \
		    -e 's/ $//' \
		    -e '/^$/d' \
		    $script_file \
		| md5sum | sed -e 's/ .*$//'
	else
		echo "FILE '$script_file' NOT FOUND at $task_start_date"
	fi
}

recipe_id="${task_id#*.}"	# Suppress task.
task="${recipe_id%%.*}"		# Suppress the recipe_id
recipe_id="${recipe_id#*.}"	# Retain only the recipe_id

if [ ! -d "$workspace/$recipe_id" ]; then
	echo "Recipe '$recipe_id' not yet imported"
	exit 0
fi

case $task in
	populate-sysroot)
		if [ ! -d "$sysroot_loc" ]; then
			echo "Sysroot '$sysroot_loc' not yet created"
			exit 0
		fi
		recipe_state="${sysroot_loc}/${recipe_statedir}/${task_id}"
		;;
	split | package)
		recipe_state="${platform_loc}/feeds/${recipe_statedir}/${task_id}"
		;;
	*)
		recipe_state="$workspace/${recipe_id}/${recipe_statedir}/${task_id}"
		;;
esac

if [ ! -r "$recipe_state.includes" ]; then
	echo "Task '$task_id' never executed yet"
	exit 0
fi

for script in $(cat "$recipe_state.includes"); do
	compute_script_sum "$script"
done # | md5sum | sed -e 's/ .*$//' 

	
